unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;

type
  TMainForm = class(TForm)
    PollTime: TTimer;
    LastLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure PollTimeTimer(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;





implementation

{$R *.DFM}

uses
  Math, ACCES32, Registry, Clipbrd;

function Right(S: String; N: Integer): String;
var
  L: Integer;

begin
  if N = 0 then begin
    Result := '';
    Exit;
  end;
  L := Length(S);
  if N >= L then begin
    Result := S;
    Exit;
  end;
  Result := Copy(S, L - N + 1, N);
end;



type
  TPCI_COMMON_CONFIG = record
     VendorID : word;
     DeviceID : word;
     Command : word;
     Status : word;
     RevisionID : byte;
     ProgIf : byte;
     SubClass : byte;
     BaseClass : byte;
     CacheLineSize : byte;
     LatencyTimer : byte;
     HeaderType : byte;
     BIST : byte;
     BaseAddresses : Array[0..5] of longint;
     Reserved1 : Array[0..1] of longint;
     RomBaseAddress : longint;
     Reserved2 : Array[0..1] of longint;
     InterruptLine : byte;
     InterruptPin : byte;
     MinimumGrant : byte;
     MaximumLatency : byte;
  end;

const
  myKey = 'Software\PCIFIND\NTioPCI\Parameters';

var
  Base: Word;
  NumPorts: Integer;

procedure TMainForm.FormCreate(Sender: TObject);
var
  buf: array [0..63] of TPCI_COMMON_CONFIG;
  num, i: Integer;
  DriverRegistry: TRegistry;

begin
    DriverRegistry:=TRegistry.Create;
    DriverRegistry.RootKey:=HKEY_LOCAL_MACHINE;

    DriverRegistry.OpenKey(MyKey, True);
    try
        num := DriverRegistry.ReadInteger('NumDevices');
    except
        on ERegistryException do num := 0;
    end;
    if (num > 0) then
        DriverRegistry.ReadBinaryData('PCICommonConfig',buf,num*sizeof(TPCI_COMMON_CONFIG))
    ;

    Base := 0;
    NumPorts := 0;
   for i := 0 to num-1 do begin
      with Buf[i] do begin
         Base := BaseAddresses[2] AND $FFF8;
         case (DeviceID) of
            $2C58:begin
               Caption:='PCI-DIO-96C3';
               NumPorts := 4;
            end;
            $2C50, $2C70:begin
               Caption:='PCI-DIO-96CT';
               NumPorts := 4;
            end;
            $0C50:begin
               Caption:='PCI-DIO-24H';
               NumPorts := 1;
            end;
            $0C51:begin
               Caption:='PCI-DIO-24D';
               NumPorts := 1;
            end;
            $0E51:begin
               Caption:='PCI-DIO-24H(C)';
               NumPorts := 1;
            end;
            $0E52:begin
               Caption:='PCI-DIO-24D(C)';
               NumPorts := 1;
            end;
            $0E50:begin
               Caption:='PCI-DIO-24S';
               NumPorts := 1;
            end;
            $0C60:begin
               Caption:='PCI-DIO-48';
               NumPorts := 2;
            end;
            $0E60:begin
               Caption:='PCI-DIO-48S';
               NumPorts := 2;
            end;
            $0C68:begin
               Caption:='PCI-DIO-72';
               NumPorts := 3;
            end;
            $0C70:begin
               Caption:='PCI-DIO-96';
               NumPorts := 4;
            end;
            $0C69:begin
               Caption:='P104-DIO-96';
               NumPorts := 4;
            end;
            $0C78:begin
               Caption:='PCI-DIO-120';
               NumPorts := 5;
            end;
            else begin
              Continue;
            end;
         end;{case}
      end;{with}
      Break;
   end;{for}

   if NumPorts = 0 then begin
     Caption := 'No PCI DIO Card Found!';
     LastLabel.Caption := Caption;
     Application.Title := Caption;
     Color := clRed;
   end
   else begin
     PollTime.Enabled := True;
   end;
end;

function IsBin(S: String): Boolean;
var
  I: Integer;

begin
  Result := False;
  for I := 1 to Length(S) do
    if (Ord(S[I]) and $FE) <> $30 then
      Exit
  ;
  Result := True;
end;

function BinStrToInt(Bin: String): Integer;
var
  I: Integer;
begin
  Result := 0;
  for I := 1 to Length(Bin) do begin
    Result := Result shl 1;
    Result := Result or (Ord(Bin[I]) and 1);
  end;
end;

procedure TMainForm.PollTimeTimer(Sender: TObject);
var
  Buf, Subuf, ReportBuf: String;
  I, L: Integer;

begin
  Buf := Clipboard.AsText;
  if Length(Buf) = 0 then Exit; //No data.
  if Length(Buf) > 27 then Exit; //Too long, it's something else.
  if not IsBin(Buf) then Exit; //It's not for us.

  L := Ceil(Length(Buf) / 8);
  L := Min(L, NumPorts * 3);

  ReportBuf := '';
  for I := 0 to L - 1 do begin
    Subuf := Right(Buf, 8);
    SetLength(Buf, Length(Buf) - 8);
    OutPortB(Base + ((I * 4) div 3), BinStrToInt(Subuf));
    if Length(Subuf) < 8 then Subuf := StringOfChar('0', 8 - Length(Subuf)) + Subuf;
    ReportBuf := Subuf + ' ' + ReportBuf;
  end;
  Buf := '';
  Subuf := '';

  LastLabel.Caption := 'Last Data: ' + ReportBuf;
end;

end.
